pub struct CallbackEnv<'id> { /* private fields */ }Expand description
The environment handed to resource callbacks (destructor, monitor-down, …).
Implementations
Sourceimpl CallbackEnv<'_>
impl CallbackEnv<'_>
Sourcepub unsafe fn with_raw<R>(
raw: *mut Env,
f: impl for<'id> FnOnce(CallbackEnv<'id>) -> R,
) -> R
pub unsafe fn with_raw<R>( raw: *mut Env, f: impl for<'id> FnOnce(CallbackEnv<'id>) -> R, ) -> R
Enter a resource callback with a freshly branded CallbackEnv.
Safety
raw must be the live env pointer the VM supplied for this callback.