pub struct Map<'id> { /* private fields */ }Expand description
An Erlang map. Immutable — all mutations return a new map.
Implementations
Sourceimpl<'id> Map<'id>
impl<'id> Map<'id>
pub fn from_raw(raw_term: Term) -> Self
Available on crate feature
raw only.Sourcepub fn get(
self,
env: impl Env<'id>,
key: impl Term<'id>,
) -> Option<AnyTerm<'id>>
pub fn get( self, env: impl Env<'id>, key: impl Term<'id>, ) -> Option<AnyTerm<'id>>
Look up key (enif_get_map_value). None if absent.
Sourcepub fn put(
self,
env: impl Env<'id>,
key: impl Term<'id>,
value: impl Term<'id>,
) -> Map<'id>
pub fn put( self, env: impl Env<'id>, key: impl Term<'id>, value: impl Term<'id>, ) -> Map<'id>
Return a new map with key set to value (enif_make_map_put, insert
or replace).
Sourcepub fn update(
self,
env: impl Env<'id>,
key: impl Term<'id>,
value: impl Term<'id>,
) -> Option<Map<'id>>
pub fn update( self, env: impl Env<'id>, key: impl Term<'id>, value: impl Term<'id>, ) -> Option<Map<'id>>
Return a new map with key updated to value (enif_make_map_update).
None if the key is absent.
Sourcepub fn remove(self, env: impl Env<'id>, key: impl Term<'id>) -> Map<'id>
pub fn remove(self, env: impl Env<'id>, key: impl Term<'id>) -> Map<'id>
Return a new map with key removed (enif_make_map_remove). Removing a
key the map does not contain returns it unchanged.
Sourcepub fn is_map(env: impl Env<'id>, term: impl Term<'id>) -> bool
pub fn is_map(env: impl Env<'id>, term: impl Term<'id>) -> bool
Returns true if term is a map (enif_is_map).
Sourcepub fn iter(self, env: impl Env<'id>) -> MapIterator<'id> ⓘ
pub fn iter(self, env: impl Env<'id>) -> MapIterator<'id> ⓘ
Iterate (key, value) pairs in unspecified order.